/*
 * A Pedagogical Java RMI Chat application with GUI and callback.
 * Author: Manal Helal
 * Date: 22 Oct 2013
 * Description: A server interface and a class implements the remote methods: login, message, logoff, getUsersNames, 
 * and a CallBack for the methods: message, getID
 * The GUI is an applet that is inserted in a frame in a main method to run as an application as well. 
 * Basic interface controls are inserted in a GridBagLayout to be able to use constraints to merge columns and rows.
 * Much can be added to enhance the Interface and the functionality.
 * 
 * Works fine for local, not tested for remote servers, but assuming it should as the simple console based RMI Chat without callback.
 * 
 * Many clients can be connected, but once disconnect, to reconnect again will give the following problem:
 * 
 * java.rmi.ServerException: RemoteException occurred in server thread; nested exception is: 
	java.rmi.ConnectException: Connection refused to host: {xxx.xxx.xxx.xxx}; nested exception is: 
	java.net.ConnectException: Connection refused
	at sun.rmi.server.UnicastServerRef.dispatch(UnicastServerRef.java:353)
	at sun.rmi.transport.Transport$1.run(Transport.java:177)
	...
 * 
 * Possible solutions:
 * 
 * http://download.oracle.com/javase/6/docs/technotes/guides/rmi/codebase.html
 * 
 * http://stackoverflow.com/questions/936119/java-rmi-connection-to-server/2417745#2417745
 * 
 * http://stackoverflow.com/questions/9531158/java-rmi-serverexception-remoteexception-occurred-in-server-thread
 * 
 * Another example online using sockets to guarantee low level connectivity:
 * 
 * http://blog.aslingga.com/2010/03/27/simple-chatting-application-using-java-rmi/
 * 
 * 
 * */

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Scanner;
import java.util.UUID;

import javax.swing.*;




public class Client extends JApplet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	static Scanner in;
	public UUID ID;
	public String name;
	Registry registry;
	ServerInterface server;  		// Server Remote Object
	CallBackImpl callBackControl;	// Client CallBack Object
	boolean connected;
	int usersCount;
	
	/* User Controls*/
	private JLabel jlblName = new JLabel("Name:"); 
	private JTextField jtfName = new JTextField(10); 
	private JLabel jlblServer = new JLabel("Enter Server IP:"); 
	private JTextField jtfServer = new JTextField(10); 
	private JButton jbtnConnect = new JButton("Connect");
	private JButton jbtnDisconnect = new JButton("Disconnect");
	private JButton jtbnUsers = new JButton("Get Users");
	private JTextArea jtaMessages = new JTextArea(100, 100);
	private JTextField jtfMsg = new JTextField(90); 
	private JButton jtbnSend= new JButton("Send");
	private JLabel jlblStatus = new JLabel(); 

	/*Applet Initialistion*/
	public void init() {
		JPanel jPanel1 = new JPanel();
		
		jPanel1.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 0;
		jPanel1.add(jlblName, c);
		c.gridx = 1;
		c.ipadx = 100;
		jPanel1.add(jtfName, c);
		c.gridx = 2;
		c.ipadx = 10;
		jPanel1.add(jlblServer, c);
		c.ipadx = 0;
		c.gridx = 3;
		c.ipadx = 100;
		jPanel1.add(jtfServer, c);
		try {
			callBackControl = new CallBackImpl(this);
		} catch (RemoteException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		jbtnConnect.addActionListener( new ActionListener () {
			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				try {	
					registry = LocateRegistry.getRegistry(jtfServer.getText());
					server = (ServerInterface) registry.lookup("ServerImplementation");
					name = jtfName.getText();
					ID = UUID.randomUUID();
					server.login((CallBack) callBackControl, name);
					connected = true;
					jbtnDisconnect.setEnabled(true);
					jtbnUsers.setEnabled(true);
					jtfMsg.setEditable(true);
					jtbnSend.setEnabled(true);
					jtaMessages.setEditable(true);
					jtbnSend.setEnabled(true);

				} catch (RemoteException | NotBoundException ex) {
					// TODO Auto-generated catch block
					ex.printStackTrace();
				}
			}}
		);
		c.gridx = 4;
		c.ipadx = 0;
		jPanel1.add(jbtnConnect, c);
		jbtnDisconnect.setEnabled(false);
		jbtnDisconnect.addActionListener( new ActionListener () {
			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				disconnect () ;
			}}
		);
		c.gridx = 5;
		jPanel1.add(jbtnDisconnect, c);
		jtbnUsers.addActionListener( new ActionListener () {
			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				if (connected && server != null)			
					try {
						ArrayList<String> users = server.getUsersNames();
						String usersStr = users.get(0);
						usersCount = users.size();
						for (int i = 1; i< usersCount;i++) 
							usersStr = usersStr.concat(", " + users.get(i));
						
						JOptionPane.showMessageDialog(null, usersStr, "Users Currently Connected", JOptionPane.INFORMATION_MESSAGE);
						jlblStatus.setText("" + usersCount + " are in chat room currently!");
					} catch (RemoteException ex) {
						// TODO Auto-generated catch block
						ex.printStackTrace();
					} 
			}}
		);
		c.gridx = 6;
		jPanel1.add(jtbnUsers, c);
		c.ipadx = 300;
		c.ipady = 300;
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 7;      	//span 6 columns
		c.gridheight = 3;      //span 3 rows
		jPanel1.add(jtaMessages, c);
		c.ipadx = 0;
		c.ipady = 0;
		c.gridx = 0;
		c.gridy = 303;
		c.gridwidth = 6;      //span 5 columns
		c.gridheight = 1;     //span 1 rows
		c.ipadx = 10;
		jPanel1.add(jtfMsg, c);
		jbtnDisconnect.setEnabled(false);
		jtbnUsers.setEnabled(false);
		jtfMsg.setEditable(false);
		jtbnSend.setEnabled(false);
		jtaMessages.setEditable(false);
		jtbnSend.setEnabled(false);
		jtbnSend.addActionListener( new ActionListener () {

			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				if (connected && server != null)
					try {
						server.message (name, jtfMsg.getText());
					} catch (RemoteException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
			}
		});
		c.ipadx = 0;
		c.gridwidth = 1;      //span 1 columns
		c.gridx = 6;
		jPanel1.add(jtbnSend, c);
		c.gridx = 0;
		c.gridy = 305;
		c.gridwidth = 7;      //span 1 columns
		jPanel1.add(jlblStatus, c);
		add(jPanel1);
		setSize(700, 500);
		
	}
	
	

	/*Client CallBack method: message*/
	void message(String msg) {
		jtaMessages.append(msg + "\n");
		System.out.println(msg);
	}

	/*Applet Closing event to disconnect Gracefully from RMI Server and remove user from server list*/
	public void stop() {
    	System.out.println("Disconnecting ...");
		disconnect () ;
	}
	
	/*Disconnect method to be called from applet closing and frame closing*/
	boolean disconnect () {
		if (connected && server != null) {
			try {
				// to log off and notify the server and hence other clients.
				server.logoff (ID, name);
				connected = false;
				jbtnDisconnect.setEnabled(false);
				jtbnUsers.setEnabled(false);
				jtfMsg.setEditable(false);
				jtbnSend.setEnabled(false);
				jtaMessages.setEditable(false);
				jtbnSend.setEnabled(false);
				
				server = null;
				callBackControl = null;
	        	System.out.println("Disconnected Gracefully!");
	        	return true;
				
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    		System.out.println("Failed to Disconnect Gracefully!");
        	return false;
		}
   		System.out.println("Already Disconnected!");
		return true;
	}
	
	/*Frame Closing event to disconnect Gracefully from RMI Server and remove user from server list*/
	void addCloseCode (JFrame frame) {
 		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener( new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent we) {
            	System.out.println("Disconnecting ...");
            	disconnect ();
                System.exit(0);
            }
        } );
	}
	
	/*Main method to be able to run as application also*/
	public static void main (String[] args) {
		Client client = new Client();
		JFrame frame = new JFrame(); 
		frame.setTitle("Chat Client"); 
		frame.add(client, BorderLayout.CENTER); 
		frame.setSize(700, 500);
		client.init(); 
		frame.setLocationRelativeTo(null);
		client.addCloseCode (frame);
		frame.setVisible(true); 
		
	}

}
